# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging.config
import os
import unittest
import uuid

from marquez_client import Clients
from marquez_client.models import (DatasetType, JobType)
from marquez_client.utils import Utils

_NAMESPACE = "my-namespace"
log = logging.getLogger(__name__)


class TestMarquezWriteOnlyClientFile(unittest.TestCase):
    def setUp(self):
        log.debug("MarquezWriteOnlyClient.setup(): ")

        os.environ['MARQUEZ_BACKEND'] = 'file'
        self.client_wo_file = Clients.new_write_only_client()
        log.info("created marquez_client_wo_file.")

    def test_create_namespace(self):
        log.info("test_create_namespace()")

        owner_name = "me"
        description = "my namespace for testing."

        self.client_wo_file.create_namespace(
            _NAMESPACE, owner_name, description)

    def test_create_dataset(self):
        dataset_name = "my-dataset"
        description = "My dataset for testing."

        fields = [
            {
                "name": "flight_id",
                "type": "INTEGER",
                "description": "flight id"
            },
            {
                "name": "flight_name",
                "type": "VARCHAR",
                "description": "flight name"
            },
            {
                "name": "flight_date",
                "type": "TIMESTAMP",
                "description": "flight date"
            }
        ]

        self.client_wo_file.create_dataset(
            namespace_name=_NAMESPACE,
            dataset_name=dataset_name,
            dataset_type=DatasetType.DB_TABLE,
            run_id=str(uuid.uuid4()),
            physical_name=dataset_name,
            source_name='my-source',
            description=description,
            schema_location=None,
            fields=fields,
            tags=None
        )

    def test_create_datasource(self):
        source_name = "flight_schedules_db"
        source_type = 'POSTGRESQL'
        source_url = "jdbc:postgresql://localhost:5432/test?" \
                     "user=fred&password=secret&ssl=true"
        description = "PostgreSQL - flight schedules database"

        self.client_wo_file.create_source(
            source_name=source_name,
            source_type=source_type,
            connection_url=source_url,
            description=description)

    def test_create_job(self):
        job_name = "my-job"
        input_dataset = [
            {
                "namespace": "my-namespace",
                "name": "public.mytable"
            }
        ]
        output_dataset = {
            "namespace": "my-namespace",
            "name": "public.mytable"
        }

        location = "https://github.com/my-jobs/blob/" \
                   "07f3d2dfc8186cadae9146719e70294a4c7a8ee8"

        context = {
            "SQL": "SELECT * FROM public.mytable;"
        }

        self.client_wo_file.create_job(
            namespace_name=_NAMESPACE,
            job_name=job_name,
            job_type=JobType.BATCH,
            location=location,
            input_dataset=input_dataset,
            output_dataset=output_dataset,
            context=context
        )

    def test_create_job_run(self):
        run_id = str(uuid.uuid4())
        job_name = "my-job"
        run_args = {
            "email": "me@mycorp.com",
            "emailOnFailure": "true",
            "emailOnRetry": "true",
            "retries": "1"
        }

        self.client_wo_file.create_job_run(
            namespace_name=_NAMESPACE,
            job_name=job_name,
            run_id=run_id,
            nominal_start_time=None,
            nominal_end_time=None,
            run_args=run_args,
            mark_as_running=True
        )

    def test_mark_job_run_as_started(self):
        run_id = str(uuid.uuid4())
        started_at = Utils.utc_now()

        self.client_wo_file.mark_job_run_as_started(
            run_id=run_id, at=started_at)

    def test_mark_job_run_as_completed(self):
        run_id = str(uuid.uuid4())
        completed_at = Utils.utc_now()

        self.client_wo_file.mark_job_run_as_completed(
            run_id=run_id, at=completed_at)

    def test_mark_job_run_as_failed(self):
        run_id = str(uuid.uuid4())
        failed_at = Utils.utc_now()

        self.client_wo_file.mark_job_run_as_failed(
            run_id=run_id, at=failed_at)

    def test_mark_job_run_as_aborted(self):
        run_id = str(uuid.uuid4())
        aborted_at = Utils.utc_now()

        self.client_wo_file.mark_job_run_as_aborted(
            run_id=run_id, at=aborted_at)


if __name__ == '__main__':
    unittest.main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging.config
import os
import unittest

from marquez_client import Clients
from marquez_client.log_backend import LogBackend

_NAMESPACE = "my-namespace"
log = logging.getLogger(__name__)


class TestMarquezWriteOnlyClientLog(unittest.TestCase):
    def test_log_backend(self):
        log.debug("MarquezWriteOnlyClient.setup(): ")
        os.environ['MARQUEZ_BACKEND'] = 'log'
        self.client_wo_log = Clients.new_write_only_client()
        log.info("created marquez_client_wo_log.")
        if not isinstance(self.client_wo_log._backend, LogBackend):
            raise Exception("Not a LogBackend.")


if __name__ == '__main__':
    unittest.main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum


class DatasetId:
    def __init__(self, namespace: str, name: str):
        self.namespace = namespace
        self.name = name


class JobId:
    def __init__(self, namespace: str, name: str):
        self.namespace = namespace
        self.name = name


class DatasetType(Enum):
    DB_TABLE = "DB_TABLE"
    STREAM = "STREAM"


class JobType(Enum):
    BATCH = "BATCH"
    STREAM = "STREAM"
    SERVICE = "SERVICE"


class RunState(Enum):
    NEW = 'NEW'
    RUNNING = 'RUNNING'
    COMPLETED = 'COMPLETED'
    FAILED = 'FAILED'
    ABORTED = 'ABORTED'



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests

from marquez_client import errors
from marquez_client.backend import Backend
from marquez_client.utils import Utils
from marquez_client.constants import API_PATH_V1


class HttpBackend(Backend):
    def __init__(self, url, timeout, api_key: str = None):
        self._timeout = timeout
        self._api_base = f"{url}{API_PATH_V1}"
        self._api_key = api_key

    def put(self, path, headers, payload):
        if self._api_key:
            Utils.add_auth_to(headers, self._api_key)

        response = requests.put(
            url=f"{self._api_base}{path}",
            headers=headers,
            json=payload,
            timeout=self._timeout
        )

        return self._response(response, as_json=True)

    def post(self, path, headers, payload=None):
        if self._api_key:
            Utils.add_auth_to(headers, self._api_key)

        response = requests.post(
            url=f"{self._api_base}{path}",
            headers=headers,
            json=payload,
            timeout=self._timeout
        )

        return self._response(response, as_json=True)

    def _response(self, response, as_json):
        try:
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            self._raise_api_error(e)

        return response.json() if as_json else response.text

    def _raise_api_error(self, e):
        # TODO: https://github.com/MarquezProject/marquez-python/issues/55
        raise errors.APIError() from e



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from unittest.mock import patch, MagicMock

import pytest
from marquez_client.utils import Utils
from marquez_client.models import DatasetType, JobType, RunState
import time


def test_mk_fields_from():
    fields_name_error = [{}]
    fields_type_error = [{"name": "flight_id"}]
    fields_valid = [
        {
            "name": "flight_id",
            "type": "integer",
            "description": "flight id",
            "tags": ["tag1", "tag2"]
        },
        {
            "name": "flight_name",
            "type": "VARCHAR",
            "description": "flight name",
            "tags": ["tag3", "tag4"]
        },
        {
            "name": "flight_date",
            "type": "timestamp",
            "description": "flight date"
        }
    ]
    new_fields_valid = [
        {
            "name": "flight_id",
            "type": "INTEGER",
            "description": "flight id",
            "tags": ["tag1", "tag2"]
        },
        {
            "name": "flight_name",
            "type": "VARCHAR",
            "description": "flight name",
            "tags": ["tag3", "tag4"]
        },
        {
            "name": "flight_date",
            "type": "TIMESTAMP",
            "description": "flight date"
        }
    ]
    assert Utils.mk_fields_from(fields=fields_valid) == new_fields_valid
    with pytest.raises(ValueError):
        Utils.mk_fields_from(fields=fields_name_error)
    with pytest.raises(ValueError):
        Utils.mk_fields_from(fields=fields_type_error)


def test_is_none():
    with pytest.raises(ValueError):
        Utils.is_none(None, None)


def test_check_name_length():
    with pytest.raises(ValueError):
        Utils.check_name_length(variable_value='a'*65,
                                variable_name='namespace_name')
    with pytest.raises(ValueError):
        Utils.check_name_length(variable_value='a'*65,
                                variable_name='owner_name')
    with pytest.raises(ValueError):
        Utils.check_name_length(variable_value='a'*65,
                                variable_name='source_name')
    with pytest.raises(ValueError):
        Utils.check_name_length(variable_value='a'*256,
                                variable_name='dataset_name')
    with pytest.raises(ValueError):
        Utils.check_name_length(variable_value='a'*256,
                                variable_name='field_name')
    with pytest.raises(ValueError):
        Utils.check_name_length(variable_value='a'*256,
                                variable_name='job_name')
    with pytest.raises(ValueError):
        Utils.check_name_length(variable_value='a'*256,
                                variable_name='tag_name')


def test_is_valid_uuid():
    with pytest.raises(ValueError):
        Utils.is_valid_uuid(variable_value='not-uuid',
                            variable_name='var_name')


def test_is_instance_of():
    with pytest.raises(ValueError):
        Utils.is_instance_of(variable_value=JobType.BATCH,
                             variable_enum_type=DatasetType)
    with pytest.raises(ValueError):
        Utils.is_instance_of(
            variable_value=DatasetType.DB_TABLE, variable_enum_type=JobType)
    with pytest.raises(ValueError):
        Utils.is_instance_of(variable_value=JobType.BATCH,
                             variable_enum_type=RunState)


@patch("time.time", MagicMock(return_value=1500100900))
def test_now_ms():
    assert int(round(time.time() * 1000)) == Utils.now_ms()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from marquez_client import Clients
from marquez_client.http_backend import HttpBackend
from marquez_client.file_backend import FileBackend
from marquez_client.log_backend import LogBackend
from marquez_client.utils import Utils
from marquez_client.constants import DEFAULT_MARQUEZ_URL, API_PATH_V1

API_KEY = 'PuRx8GT3huSXlheDIRUK1YUatGpLVEuL'
API_BASE = f"{DEFAULT_MARQUEZ_URL}{API_PATH_V1}"


def test_new_client():
    os.environ['MARQUEZ_API_KEY'] = API_KEY

    from marquez_client.client import _USER_AGENT, _HEADERS
    headers_with_auth = {'User-Agent': _USER_AGENT}

    # Add API key to headers
    Utils.add_auth_to(headers_with_auth, API_KEY)

    client = Clients.new_client()
    assert client._api_base == API_BASE
    assert _HEADERS == headers_with_auth

    del os.environ['MARQUEZ_API_KEY']


def test_new_wo_client_http():
    os.environ['MARQUEZ_BACKEND'] = 'http'
    os.environ['MARQUEZ_API_KEY'] = API_KEY

    client = Clients.new_write_only_client()
    assert isinstance(client._backend, HttpBackend)
    assert client._backend._api_base == API_BASE
    assert client._backend._api_key == API_KEY

    del os.environ['MARQUEZ_BACKEND']
    del os.environ['MARQUEZ_API_KEY']


def test_new_wo_client_file():
    os.environ['MARQUEZ_BACKEND'] = 'file'

    client = Clients.new_write_only_client()
    assert isinstance(client._backend, FileBackend)

    del os.environ['MARQUEZ_BACKEND']


def test_new_wo_client_log():
    os.environ['MARQUEZ_BACKEND'] = 'log'

    client = Clients.new_write_only_client()
    assert isinstance(client._backend, LogBackend)

    del os.environ['MARQUEZ_BACKEND']



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
import unittest

import mock

from marquez_client.log_backend import LogBackend
from marquez_client.utils import Utils

log = logging.getLogger(__name__)


class TestLogBackend(unittest.TestCase):
    def setUp(self):
        log.debug("TestLogBackend.setup(): ")
        self.log_backend = LogBackend()
        self.put_json = Utils.get_json("tests/resources/put.json")
        self.post_json = Utils.get_json("tests/resources/post.json")

    @mock.patch("json.dumps")
    def test_put(self, mock_put):
        log.debug("TestLogBackend.test_put(): ")

        path = self.put_json["path"]
        headers = self.put_json["headers"]
        json_payload = self.put_json["payload"]

        self.log_backend.put(path, headers, json)
        mock_put.return_value = self.put_json

        assert path == mock_put.return_value["path"]
        assert headers == mock_put.return_value["headers"]
        assert json_payload == mock_put.return_value["payload"]

    @mock.patch("json.dumps")
    def test_post(self, mock_post):
        log.debug("TestLogBackend.test_post(): ")

        path = self.post_json["path"]
        headers = self.post_json["headers"]
        json_payload = self.post_json["payload"]

        self.log_backend.post(path, headers, json)
        mock_post.return_value = self.post_json

        assert path == mock_post.return_value["path"]
        assert headers == mock_post.return_value["headers"]
        assert json_payload == mock_post.return_value["payload"]


if __name__ == '__main__':
    unittest.main()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

DEFAULT_TIMEOUT_MS = 10000
DEFAULT_NAMESPACE_NAME = 'default'
DEFAULT_MARQUEZ_BACKEND = 'http'
DEFAULT_MARQUEZ_URL = 'http://localhost:5000'
DEFAULT_MARQUEZ_FILE = '/tmp/marquez/client.requests.log'
DEFAULT_LIMIT = 100
DEFAULT_OFFSET = 0

API_PATH_V1 = '/api/v1'



#!/usr/bin/env python
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = [
    "requests",
    "six",
    "pyrfc3339",
]

extras_require = {
    "tests": ["pytest", "pytest-cov", "mock", "flake8"],
}
extras_require["dev"] = set(sum(extras_require.values(), []))

setup(
    name="marquez-python",
    version="0.16.1rc1",
    description="Marquez Python Client",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Marquez Project",
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    python_requires=">=3.6",
    zip_safe=False,
    keywords="marquez",
)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pytest
import mock

from marquez_client import MarquezWriteOnlyClient
from marquez_client.models import (DatasetType, JobType)
from marquez_client.utils import Utils


@pytest.fixture
def wo_client():
    return MarquezWriteOnlyClient(backend=mock.Mock())


def test_create_namespace(wo_client):
    wo_client.create_namespace(
        namespace_name='test-namespace',
        owner_name='test-owner'
    )

    wo_client._backend.put.assert_called_once_with(
        path='/namespaces/test-namespace',
        headers=mock.ANY,
        payload={'ownerName': 'test-owner'}
    )


def test_create_source(wo_client):
    wo_client.create_source(
        source_name='test-source',
        source_type='postgresql',
        connection_url='postgresql://localhost:5432/testdb'
    )

    wo_client._backend.put.assert_called_once_with(
        path='/sources/test-source',
        headers=mock.ANY,
        payload={
            'type': 'POSTGRESQL',
            'connectionUrl':
                'postgresql://localhost:5432/testdb'
        }
    )


def test_create_dataset(wo_client):
    wo_client.create_dataset(
        namespace_name='test-namespace',
        dataset_name='test-dataset',
        dataset_type=DatasetType.DB_TABLE,
        physical_name='test-physical-dataset',
        source_name='test-source',
        fields=[
            {'name': 'field0', 'type': 'int4'},
            {'name': 'field1', 'type': 'varchar'},
            {'name': 'field2', 'type': 'varchar'},
        ]
    )

    wo_client._backend.put.assert_called_once_with(
        path='/namespaces/test-namespace/datasets/test-dataset',
        headers=mock.ANY,
        payload={
            'type': 'DB_TABLE',
            'physicalName': 'test-physical-dataset',
            'sourceName': 'test-source',
            'fields': [
                {'name': 'field0', 'type': 'INT4'},
                {'name': 'field1', 'type': 'VARCHAR'},
                {'name': 'field2', 'type': 'VARCHAR'}
            ]
        }
    )


def test_create_job(wo_client):
    wo_client.create_job(
        namespace_name='test-namespace',
        job_name='test-job',
        job_type=JobType.BATCH,
        input_dataset=[{
            'namespace': 'test-namespace',
            'name': 'public.test_table'
        }],
        output_dataset=[],
        location=('https://github.com/repo/test/commit/' +
                  '3c34a757fb501f4ee0e297a6ce2888136e495c70'),
        context={
            'sql': 'SELECT * FROM test_table;'
        }
    )

    wo_client._backend.put.assert_called_once_with(
        path='/namespaces/test-namespace/jobs/test-job',
        headers=mock.ANY,
        payload={
            'inputs': [{
                'namespace': 'test-namespace',
                'name': 'public.test_table'
            }],
            'outputs': [],
            'type': 'BATCH',
            'context': {
                'sql': 'SELECT * FROM test_table;'
            },
            'location': ('https://github.com/repo/test/commit/' +
                         '3c34a757fb501f4ee0e297a6ce2888136e495c70')
        }
    )


def test_create_job_run(wo_client):
    wo_client.create_job_run(
        namespace_name='test-namespace',
        job_name='test-job',
        run_id='71feb41b-be50-428c-8470-37b9c292f787'
    )

    wo_client._backend.post.assert_called_once_with(
        path='/namespaces/test-namespace/jobs/test-job/runs',
        headers=mock.ANY,
        payload={
            'id': '71feb41b-be50-428c-8470-37b9c292f787'
        }
    )


def test_mark_job_run_as_started(wo_client):
    started_at = Utils.utc_now()

    wo_client.mark_job_run_as_started(
        run_id='71feb41b-be50-428c-8470-37b9c292f787',
        at=started_at
    )

    wo_client._backend.post.assert_called_once_with(
        path=MarquezWriteOnlyClient._path(
            '/jobs/runs/71feb41b-be50-428c-8470-37b9c292f787/start?at={0}',
            started_at
        ),
        headers=mock.ANY
    )


def test_mark_job_run_as_completed(wo_client):
    completed_at = Utils.utc_now()

    wo_client.mark_job_run_as_completed(
        run_id='71feb41b-be50-428c-8470-37b9c292f787',
        at=completed_at
    )

    wo_client._backend.post.assert_called_once_with(
        path=MarquezWriteOnlyClient._path(
            '/jobs/runs/71feb41b-be50-428c-8470-37b9c292f787/complete?at={0}',
            completed_at
        ),
        headers=mock.ANY
    )


def test_mark_job_run_as_failed(wo_client):
    failed_at = Utils.utc_now()

    wo_client.mark_job_run_as_failed(
        run_id='71feb41b-be50-428c-8470-37b9c292f787',
        at=failed_at
    )

    wo_client._backend.post.assert_called_once_with(
        path=MarquezWriteOnlyClient._path(
            '/jobs/runs/71feb41b-be50-428c-8470-37b9c292f787/fail?at={0}',
            failed_at
        ),
        headers=mock.ANY
    )


def test_mark_job_run_as_aborted(wo_client):
    aborted_at = Utils.utc_now()

    wo_client.mark_job_run_as_aborted(
        run_id='71feb41b-be50-428c-8470-37b9c292f787',
        at=aborted_at
    )

    wo_client._backend.post.assert_called_once_with(
        path=MarquezWriteOnlyClient._path(
            '/jobs/runs/71feb41b-be50-428c-8470-37b9c292f787/abort?at={0}',
            aborted_at
        ),
        headers=mock.ANY
    )


def test_path():
    path = MarquezWriteOnlyClient._path('/namespaces/{0}', 'test-namespace')
    assert path == '/namespaces/test-namespace'



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from marquez_client import MarquezClient, MarquezWriteOnlyClient
from marquez_client.http_backend import HttpBackend
from marquez_client.file_backend import FileBackend
from marquez_client.log_backend import LogBackend
from marquez_client.utils import Utils
from marquez_client.constants import (
    DEFAULT_MARQUEZ_BACKEND,
    DEFAULT_MARQUEZ_URL,
    DEFAULT_MARQUEZ_FILE,
    DEFAULT_TIMEOUT_MS
)


# Marquez Clients
class Clients(object):
    @staticmethod
    def new_client():
        return MarquezClient(
            url=os.environ.get('MARQUEZ_URL', DEFAULT_MARQUEZ_URL),
            api_key=os.environ.get('MARQUEZ_API_KEY')
        )

    @staticmethod
    def new_write_only_client():
        return MarquezWriteOnlyClient(
            backend=Clients._backend_from_env(),
        )

    @staticmethod
    def _backend_from_env():
        backend = \
            os.environ.get('MARQUEZ_BACKEND', DEFAULT_MARQUEZ_BACKEND).upper()

        if backend == 'HTTP':
            url = os.environ.get('MARQUEZ_URL', DEFAULT_MARQUEZ_URL)
            api_key = os.environ.get('MARQUEZ_API_KEY')
            timeout = Utils.to_seconds(
                os.environ.get('MARQUEZ_TIMEOUT_MS', DEFAULT_TIMEOUT_MS))
            return HttpBackend(url, timeout, api_key)
        elif backend == 'FILE':
            file = os.environ.get('MARQUEZ_FILE', DEFAULT_MARQUEZ_FILE)
            return FileBackend(file)
        elif backend == 'LOG':
            return LogBackend()



import pytest

from marquez_client.backend import Backend


def test_put():
    with pytest.raises(NotImplementedError):
        Backend().put(path='', headers='', payload='')


def test_post():
    with pytest.raises(NotImplementedError):
        Backend().post(path='', headers='')
